// sam IDE drive operating system 

#define hdiodebug 0

#include <string.h>
#include "hmalloc.h"
#include "hsamio.h"
#include "hdstr.h"
#include "hdio.h"
#include "hlow.h"
#define retry -2

// global vars with initalisers (stored in 'rom' or external to program)
volatile port uchar hddport @ 189;
volatile port uchar hdrport @ 191;
uchar  init_done;
uchar  writeverify = 0; // set to 1 to verify writes;
uchar  retrycnt;

// vars with standard types
uchar portdata;
uint  hdmaxtrk;
uchar hdmaxhead;
uchar hdmaxsect;

// vars of structures


// procedures
void waitforrdy()
{
  do
  { hdrport = 0xF7;
    portdata=hddport;
  } while (portdata > 0x7F);
}

	
void hdinit()
{
 s_label *mylabel;
 uint dummy;
 
 if (init_done == 0)
 {
  init_done = 1;
//  hdrport = 0xDF;	// send hardware reset
//  dummy   = hddport;
//  pause(20);
  
  hdrport = 0xEE;	// send software reset	
  hddport = 0x0E;
  hddport = 0x0;
  pause(20);

  hddport = 0xA;	// clear software reset
  hddport = 0x0;
  pause(20);

  hdrport = 0x0FF;	// clear control reg on interface
  dummy   = hddport;
  dummy   = hddport;
  pause(20);

  waitforrdy();

  // assuming the disk has a label load its size info  
  if ((mylabel = calloc(1,sizeof(s_dir))) != 0)
  {
    hdread((char*)mylabel,0L);
    hdmaxsect = mylabel->l_sects;    
    hdmaxhead = mylabel->l_heads;
    hdmaxtrk  = mylabel->l_tracks;
    free(mylabel);
  } 
 }
}


void hdread(void *hddata, long se)
// read sector se into buffer hddata
// hddata must be at least 512 bytes long
{
	retrycnt=10;
	do
	{
	 hdsetup(se);
	 hdrport = 0xF7;
	 hddport = 0x20;
	 hddport = 0x0;
	 waitforrdy();
	 hdrport = 0xF0;
	 rdbuf(hddata);
	} while (hdchkrdy()==retry);
}


void hdwrite(void *hddata, long se)
// write sector se with data from hddata
{
//	if (se == 0) hderror(86); // attempt to write to sector 0
	
	retrycnt=10;
	do
	{
	 hdsetup(se);
	 hdrport = 0xF7;
	 hddport = 0x31;
	 hddport = 0x0;
	 waitforrdy();
	 hdrport = 0xF0;
	 wrbuf(hddata);
	 waitforrdy();
	 hdsetup(se);
	 hdrport = 0xF7;
	 hddport = 0x41;
	 hddport = 0x0;
	 waitforrdy();
  } while (hdchkrdy()==retry);
 
}
 

int hdchkrdy()
{
 uint statval,dummy;

 hdrport = 247;
 statval = hddport;
 dummy  = hddport;
 if ((statval == 0x50) || (statval == 0x52)) return(ok);
 if ((statval & 0x8) && (retrycnt--)) return(retry);

 hderror(85);
// report disk error
 //  printf("\rbad status= &%x\r",status);
 // 
 //  hdrport = 0xF1;
 //  status  = hddport;
 //  dummy   = hddport;
 //  printf("error=&%x",status);
 // 
 //  hdrport = 0xf5;
 //  status  = hddport;
 //  dummy   = hddport;
 //  track   = status*256;
 //  hdrport = 0xf4;
 //  status  = hddport;
 //  dummy   = hddport;
 //  track += status;
 //  printf(" track=%d",track);
 // 
 // 
 //  hdrport =0xf6;
 //  status  = hddport;
 //  dummy   = hddport;
 //  printf(" head=%d",(status & 0x0f));
 // 
 //  hdrport =0xf3;
 //  status  =hddport;
 //  dummy   =hddport;
 //  printf(" sect=%d\r",status);

 // force drive diagnostic
 //  hdrport=0xF7;
 //  hddport=0x90;
 //  hddport=0x0;
 //  waitforrdy();
 //  hdchkrdy();
 return(error);
 
}


void hdsetup(long se)
{
 uchar trhi,trlo,head,sect;

 sect = se % hdmaxsect;
 se   /=     hdmaxsect;
 head = se % hdmaxhead;
 se   /=     hdmaxhead;
 trlo = se % 256;
 trhi = se / 256;

 sect++;
 head &= 0x1F;
 head |= 0xA0;

 hdrport = 243;
 hddport = sect;
 hddport = 0;

 hdrport = 244;
 hddport = trlo;
 hddport = 0;

 hdrport = 245;
 hddport = trhi;
 hddport = 0;

 hdrport = 246;
 hddport = head;
 hddport = 0;

 hdrport = 242;
 hddport = 1;
 hddport = 0;

}


void rdbuf(void *buf)
{ // buf passed in DE reg
#asm
 EX DE,HL
; DI
 LD BC,189
 INIR
 INIR
; EI
#endasm
}


void wrbuf(void *buf)
{ // buf passed in DE reg
#asm
 EX DE,HL
; DI
 LD BC,189
 OTIR
 OTIR
; EI
#endasm
}
