
#include "hdstr.h"
#include "hmalloc.h"
#include "hdio.h"
#include <stdio.h>
#include <string.h>
#include "hdir.h"
#include "hfree.h"
#include "hfname.h"
#include "hlow.h"

uint currdir=0;


// **************************************************************************
// directory handling routines
// **************************************************************************


s_fhead* getnextfreedir(s_dir *mydir)
// return a pointer to the next free dir entry using the buffer provided
// dirbuf will hold the dir sector
// directory will be extended if full 
{
 long  se;
 uint   i;
 
 se=DIRSTART;  // start of directory
 do
 {
  flushdir(mydir);
  hdread(mydir,se);
  se = mydir->d_next;
  for (i=0;i<n_head;i++)
  {
   if (mydir->d_head[i].h_fsam == 0L)
   {
    return(&mydir->d_head[i]);
   }
  }
 } while (se);
 extenddir();
 return(getnextfreedir(mydir));
}

void extenddir()
// extend the directory by one more sector
//
{
 long nextse,thisse;
 s_dir *mydir;

 nextse = getnextsector();
 if ((mydir = calloc(1,sizeof(s_dir))) == 0)
 {
	 hderror(OUTOFMEMORY);
 }
 memset(mydir,0,sizeof(s_dir));
 mydir->d_this = nextse;
 mydir->d_dirty=dirty;

 flushdir(mydir);

 // now make last existing dir entry point to the new one.
 thisse = DIRSTART;
 do
 {
  hdread(mydir,thisse);
  thisse = mydir->d_next;
 } while (thisse);

 mydir->d_next = nextse;
 mydir->d_dirty = dirty;
 flushdir(mydir);
 free(mydir);
 
}

long finddirentry(char *filename, int dirnum)
// return the sector address of the fsam of the file
// with name pointed to by filename and in the
// directory with number dirnum
// return error if not found
{
 long se;
 uint i;
 char fname[11];
 s_dir *mydir;
 
#if hdosdebug
 printf("\rfinddirentry looking for \"%s\" in dir %d",filename,dirnum);
#endif
 
 strncpy(fname,filename,namelen);
 padname(fname);
 if ((mydir = calloc(1,sizeof(s_dir))) == 0) hderror(OUTOFMEMORY);

 se=DIRSTART;  // start of directory
 do
 {
  hdread(mydir,se);
  se = mydir->d_next;
  for (i=0;i<n_head;i++)
  {
   if (mydir->d_head[i].h_dir == dirnum)
   {
    if ((filenamecmp(mydir->d_head[i].h_name,fname))==ok)
    {
#if hdosdebug
 printf("\rfinddirentry found ok");
#endif	    
     free(mydir);
     return(mydir->d_head[i].h_fsam);
    }
   }
  }
 } while (se);
#if hdosdebug
 printf("\rfinddirentry not found");
#endif
 free(mydir);
 return(error);
}

int getdirnum (char* s1)
// given a string read down it to get the dir number of the path part
// return the dir number or error if not found 
// the given string must be null terminated
{
 char *pp,dirname[11];
 int  tmpdir;
 long se;
 s_fsam * myfsam;

#if hdosdebug
 printf("\rgetdirnum path = %s",s1);
#endif
 tmpdir = currdir;   // start of at the current dir
 if (*s1 == dirsep)  // path starts from root
 {
#if hdosdebug
 printf("\rgetdirnum starts from root");
#endif	 
  tmpdir = 0;
  s1++;   // skip past the starting /
 }
 
 if ((myfsam = calloc(1,sizeof(s_fsam))) == 0) hderror(OUTOFMEMORY);
	
 for (;;)
 {
  pp = dirname;	 
  memset(dirname,0,11);

  while ((*s1 != 0) && (*s1 != dirsep))
  {
   *pp++ = *s1++;
  }
#if hdosdebug
  printf("\rgetdirnum pathpart = \"%s\"",dirname);
#endif

  if (*s1 == 0)
  {
#if hdosdebug
 printf("\rgetdirnum ok. dirnum = %d",tmpdir);
#endif
    free(myfsam);
    return(tmpdir);  // finished
  }
  
  if ((se = finddirentry(dirname,tmpdir)) == error)
  {
#if hdosdebug
 printf("\rgetdirnum did not find dir \"%s\" in dirnum %d",dirname,tmpdir);
#endif
    free(myfsam);	  
    return(error);  // failed to find path
  }
  
  hdread(myfsam,se);             // get fsam for this dir

  if ((myfsam->f_fib.f_type & 0x1F) != ft_Hdir) // not a directory
  {
#if hdosdebug
 printf("\rgetdirnum \"%s\" not a directory",dirname);
#endif
    free(myfsam);
    return(error);
  }
   
  tmpdir = myfsam->f_fib.f_info.u_dirinfo.dirnum;
#if hdosdebug
  printf("\rgetdirnum now in dir #%d",tmpdir);
#endif
  s1++;
 }
}


int hddirfunc(char *path,uint action)
// return error if bad path
// return ok if files found
// return error if no matching file found
// action = 0-erase,1-hide,2-unhide,3-protect,4-unprotect,5-as 4 + dirs
// 6-set archive,7-reset archive, 8 return number of files in dir
{
 long se;
 int i,result;
 int dirnum;
 char *filename;
 s_dir *mydir;
 s_fsam *myfsam;

// get the directory number
 if ((dirnum = getdirnum(path)) == error) 
 {
#if hdosdebug
	 printf("\rhddirfunc: bad path %s",path);
#endif
  hderror(BADPATH);
 }

 // get the filename part
 filename = getfilepart(path);
 if (action==8) *filename=wild2;
#if hdosdebug
	 printf("\rhddirfunc: filepart = %s",filename);
#endif
 

 // now search the diretory to find matching files
 se=DIRSTART;  // start of directory
 result = error;
 if (action == 8) result=0;

 if ((myfsam = calloc(1,sizeof(s_fsam))) == 0) hderror(OUTOFMEMORY);
 if ((mydir = calloc(1,sizeof(s_dir))) == 0) hderror(OUTOFMEMORY);
 
 do
 {
  flushdir(mydir);
  hdread(mydir,se);
  se = mydir->d_next;
  for (i=0;i<n_head;i++)
  {
   if (mydir->d_head[i].h_dir == dirnum)  // this dir
   if (mydir->d_head[i].h_fsam != 0)      // has a fsam
   {
    if (filenamecmp(mydir->d_head[i].h_name,filename) == ok)
    {
     flushfsam(myfsam);
     hdread(myfsam,mydir->d_head[i].h_fsam);

     switch (action) 
     {
     case 0: // erase
      {
       if (!(myfsam->f_fib.f_type & ft_protected))
       {
        mydir->d_head[i].h_dir = removed;
        mydir->d_dirty = dirty;
        result = ok;
       }
       break;
      }
      
     case 8: // count 
      {
       result++;
       break;
      }
      
     default:
      { 
       switch (action)
       {
       case 1: // hide
        myfsam->f_fib.f_type |= ft_hidden;break;

       case 2: // unhide
        myfsam->f_fib.f_type &= (0xFF ^ ft_hidden);break;

       case 3: // protect
        myfsam->f_fib.f_type |= ft_protected;break;

       case 4: // unprotect except dirs
        if ((myfsam->f_fib.f_type & 0x1F) == ft_Hdir) break;

       case 5: // unprotect inc dirs
        myfsam->f_fib.f_type &= (0xFF ^ ft_protected); break;

       case 6: // set archive
        myfsam->f_fib.f_type |= ft_archive;break;

       case 7: // reset archive
        myfsam->f_fib.f_type &= (0xFF ^ ft_archive);break;

       default : printf("\rbad case in hddirfunc %d",action);
       }
       myfsam->f_dirty = dirty;
       result = ok;
      }
     }
    }
   }
  }
 } while (se);
 flushdir(mydir);
 flushfsam(myfsam);
 free(mydir);
 free(myfsam);
 return(result);
}

// **************************************************************************
// end of directory handling routines
// **************************************************************************
