
#include "hdstr.h"
#include "hdio.h"
#include "hfile.h"
#include "hmalloc.h"
#include "hlow.h"

volatile port uchar lmpr @ 250;
volatile port uchar hmpr @ 251;
volatile uchar curchan @ 0x5ab2;
volatile uchar dospage @ 0x5bc2;

// **************************************************************************
// other routines
// **************************************************************************

void samerror (uint hderrno)
// abort hdos and report error
{
  // call the rst 8 vector with the error code passed
  // will halt the program
  // errno will have been passed in regs de
#asm
	ld a,e
	ld (errnum),a
	rst 8
	errnum: defb 0
#endasm

}
	
void hderror(uint hderrno)
// report a hd error and abort hdos hi level stuff
{
	int i;

	for (i=1;i<16;i++)
	{	// close all channels
		hclose(i);
	}
	freeall();	// free all heap
	samerror(hderrno);
}

void gettime(s_time *time)
// get the current time and store in the structure passed
{

 // read datetime();
 	
 time->t_DD = (10*(peekdvar(81)-'0'))+peekdvar(82)-'0';
 time->t_MM = (10*(peekdvar(84)-'0'))+peekdvar(85)-'0';
 time->t_YY = (10*(peekdvar(87)-'0'))+peekdvar(88)-'0';
 time->t_hh = (10*(peekdvar(96)-'0'))+peekdvar(97)-'0';
 time->t_mm = (10*(peekdvar(99)-'0'))+peekdvar(100)-'0';
 time->t_ss = (10*(peekdvar(102)-'0'))+peekdvar(103)-'0';
}
 
void flushdir(s_dir *mydir)
// rewrite a dirbuf if it is dirty
{
 if (mydir->d_dirty != clean)
 {
  mydir->d_dirty = clean;
  hdwrite(mydir,mydir->d_this);
 }
}

void flushfsam(s_fsam *myfsam)
// rewrite a fsambuf if it is dirty
{ 
 if (myfsam->f_dirty != clean)
 {
  gettime(&myfsam->f_fib.f_upd);		
  myfsam->f_dirty = clean;
  hdwrite(myfsam,myfsam->f_myadr);
 }
}

void saminc(s_samadr *sadr, uint inc)
{ // increment the passed sam address by the inc which must be +ve
  if (inc > 0x4000) hderror(NUMTOOBIG);

  sadr->offset += inc;
  if (sadr->offset > 0x4000)
  {
	  sadr->page++;
	  sadr->offset &= 0x03ff;
  }
}

void samdec(s_samadr *sadr, uint dec)
{ // converse of saminc
  if (dec > 0x4000) hderror(NUMTOOBIG);

  sadr->offset -= dec;
  if (sadr->offset > 0x4000)
  {
	  sadr->page--;
	  sadr->offset &= 0x03ff;
  }
}

// **************************************************************************
// end of other routines
// **************************************************************************
//|S
// ************************************************************************
// assembled routines and intrupts
// ************************************************************************

void fastxfer(char* dest, char* src, uchar page)
{ // move 512 bytes from src to dst
  // dest in de, src in bc, page on the stack at ix+5
#asm
  di
  ld ix,0
  add ix,sp
  in a,(_lmpr) 	; get and save lmpr	
  ex af,af'
  ld a,(ix+4)	; get page number
  or 0x20	; ram in A
  and 0x3F	; rom 1 off write enable A
  out (_lmpr),a ; select required page !! stack is now out !!	
  ld l,c	; src to hl
  ld h,b
  ld bc,512	; bytes to xfer	
  ldir		; move em		
  ex af,af'	; recover original page	
  out (_lmpr),a	; orig page and stack back in
  ei
#endasm
}

char sgetc(s_samadr *sadr)
{ // peek (sadr)
  // on entry de -> sadr
  char chr;
#asm
  di
  push af
  push bc
  IN A,(_lmpr)
  ld b,a	; save lmpr
  ex de,hl
  ld a,(hl)	; get page 0-31	
  inc hl
  ld e,(hl)	; offset lobyte
  inc hl
  ld d,(hl)	; offset hibyte	
  or 0x20   	; RAM not ROM in section A
  out (_lmpr),a ; required page now in bank A 
  ;		!! don't touch stack cos it not there any more
  ld a,(de);    ; get required byte
  ld c,a        ; temp save
  ld a,b	; recover lmpr
  out (_lmpr),a	; restore orig bank A page and stack in B
  ld l,c   	; save char read ready for return
  pop bc
  pop af
  ei
  ret;
#endasm
  return(chr);  // won't ever get here 
  
}  

void sputc(s_samadr* sadr, char chr)
{ // poke sadr++,chr
  // on entry de -> sadr,  c = chr
  
#asm
  di
  push hl
  push bc	; save chr
  IN A,(_lmpr)
  ld b,a	; save lmpr
  ex de,hl
  ld a,(hl)	; get page 0-31	
  inc hl
  ld e,(hl)	; offset lobyte
  inc hl
  ld d,(hl)	; offset hibyte	
  or 0x20   	; RAM not ROM in section A
  out (_lmpr),a ; required page now in bank A 
  ;		!! don't touch stack cos it not there any more
  ld a,c        ; get chr from c
  ld (de),a;    ; poke required byte
  ld a,b	; recover lmpr
  out (_lmpr),a	; restore orig bank A page and stack in B
  pop bc	; recover chr
  pop hl
  ei
#endasm
}

void getarg(char* cmd)
{
	// get one string off the sam stack into my heap
	// cmd must be big enough or you will crash
#asm
	push iy
	push de ; save address of cmd
	call 0x124
	call 0x12a
	ex de,hl	; hl-> start of string	
	pop de		; de -> cmd
	ldir		; bc already had length
	pop iy
#endasm
}



int peekdvar (int var)
{
	// return the value of the requested dvar

	s_samadr dvar;

	dvar.offset = 0x220 + var;
	dvar.page = dospage;

	return (sgetc(&dvar));

}

void pokedvar (int var, char value)
{
	// poke a dvar

	s_samadr dvar;
	dvar.offset = 0x220 + var;
	dvar.page = dospage;
	sputc(&dvar,value);
}
