
// sam IDE drive operating system v 1.0

#include <stdio.h>
#include <string.h>
#include "hmalloc.h"
#include "hdstr.h"



// global vars with initalisers (stored in 'rom' or external to program)

// vars with standard types
char theheap[0x1200];

// vars of structures
s_heap *heapstart;


void* malloc(uint size)
// build a list 
{
  //
  s_heap *thisheap,*nextheap;

  if (heapstart == 0) // first time
  {
    heapstart = (s_heap*)theheap;
    heapstart->next = 0;			// no next heap
    heapstart->size = sizeof(theheap);		// max heap size
    heapstart->inuse= 0;			// not in use
  }

  // not first time so look for a unused mem big enough
  thisheap = heapstart;
  do
  {
   //
   if (thisheap->inuse == 0)
   if (thisheap->size >= size) // use this one
   {
    //
    thisheap->inuse = 1;

    // test if much bigger
    nextheap = (s_heap*)(((size + sizeof(s_heap))) + (uint)thisheap);
    if ((thisheap->next - nextheap) > (int)sizeof(s_heap))
    {
     // build an interim structure
     nextheap->next  = thisheap->next;
     nextheap->inuse = 0;
     nextheap->size  = thisheap->size - size - sizeof(s_heap);
     thisheap->next  = nextheap;
     thisheap->size  = size;
    }
    return(&thisheap->newmem);
   }
   thisheap = thisheap->next;
  } while (thisheap);

  // found none so return 0
  return(0);
  
}


void free(void* adr)
{
 //
 s_heap *th, *nh ,*ph;
 th = heapstart;
 ph = 0;

 do
 {
  //
  if (&th->newmem == adr)
  {
   nh = th->next;
   th->inuse = 0;
   if (nh->inuse == 0) // consolodate with next block
   {
    //
    th->size += (nh->size+sizeof(s_heap));
    th->next  = nh->next;
   }
   if (ph)
   if (ph->inuse == 0) // consolodate with prev block
   {
    //
    ph->size += (th->size+sizeof(s_heap));
    ph->next  = th->next;
   }
   return;
  }
  ph = th;
  th = th->next;
 } while (th);
}

   	    	
void* calloc(uint size, uint count)
// allocate room for count * size and clear to 0
{
  void *pp;

  if ((pp = malloc(size*count)) == 0) return(0); // no room
  memset(pp,0,(size*count));
  return(pp);
}


void* realloc(void* ptr, uint cnt)
{
  void* dst;

  if ((dst = malloc(cnt)) == 0) return(0);
  memcpy(dst,ptr,cnt);
  free(ptr);
  return(dst);
}

void freeall()
{
// completly clear the heap (seems like a good idea)
  s_heap *th;

  if (heapstart == 0) return;   
  do
  {
   for (th=heapstart;!th->inuse;th=th->next);
   if (th->inuse) free(&th->newmem);
  } while (heapstart->next);
}


void printheap()
{
// print each entry on the heap
  s_heap *th;

  if (heapstart == 0) return;   
  th = heapstart;
  do
  {
	printf("\r %4.4x >%4.4x size=%4.4x usage=%d",th,th->next,th->size,th->inuse);
	th=th->next;
  } while (th);
  printf("\r");
}


